"""deepmanager.command.dot

Implements DeepManager's 'dot' command."""

from types import IntType
from deepmanager.core import Command
from deepmanager.errors import *
from deepmanager.package import *

class dot(Command):

	description = "build a graphviz dot graph"

	user_options = [
		('rpm-db', None,
		 "use local installed database as information source"),
		('rpm-hdlist=', None,
		 "use given hdlists as information sources"),
		('pickle=', None,
		 "use given pickle file as information source"),
		('output=', 'o',
		 "output dot filename"),
		('all-deps', None,
		 "output dependencies of all packages"),
		('all-revdeps', None,
		 "output reverse dependencies of all packages"),
		('deps=', None,
		 "show depenencies from given packages or relations"),
		('revdeps=', None,
		 "show reverse dependencies from given packages or relations"),
		('exclude=', 'x',
		 "exclude given package from graph"),
		('maxdepth=', None,
		 "set maximum depth to given value"),
		('show-filedeps', None,
		 "show file depenencies explicitly"),
		('show-alldeps', None,
		 "show all depenencies explicitly"),
		('show-versions', None,
		 "show version information"),
		]
	
	boolean_options = ["rpm_db", "show_filedeps", "show_alldeps",
					   "show_versions", "all_deps", "all_revdeps"]
	
	list_options = ["rpm_hdlist", "deps", "revdeps", "exclude"]

	def initialize_options(self):
		self.rpm_db = 0
		self.rpm_hdlist = []
		self.pickle = None
		self.output = None
		self.all_deps = 0
		self.all_revdeps = 0
		self.deps = []
		self.revdeps = []
		self.exclude = []
		self.maxdepth = 9999
		self.show_filedeps = 0
		self.show_alldeps = 0
		self.show_versions = 0
	
	def finalize_options(self):
		if not (self.rpm_db or self.rpm_hdlist or self.pickle):
			raise DeepManagerOptionError, \
				  "you must specify one or more information sources"

		if not self.output:
			raise DeepManagerOptionError, \
				  "you must specify a filename for output"
		elif (self.all_deps or self.all_revdeps) \
			 and self.output.find("%(name)s") == -1:
			raise DeepManagerOptionError, \
				  "when outputing all, -o argument must be" \
				  " a template using '%(name)s'"

				  
		self.ensure_filename_list("rpm_hdlist")
		self.ensure_filename("pickle")
		self.ensure_string_list("deps")
		self.ensure_string_list("revdeps")

		if type(self.maxdepth) != IntType:
			try:
				self.maxdepth = int(self.maxdepth)
			except ValueError:
				raise DeepManagerOptionError, \
					  "invalid maxdepth value"
	
	def run(self):
		if self.rpm_db:
			pm = get_pm("rpm")
			self.announce("reading database...", noeol=1)
			self.pkgset = pm.pkgset_from_db()
			self.announce("done")

		if self.rpm_hdlist:
			pm = get_pm("rpm")
			self.announce("reading hdlists...", noeol=1)
			self.pkgset = pm.pkgset_from_hdlists(self.rpm_hdlist)
			self.announce("done")

		if self.pickle:
			self.announce("reading pickle...", noeol=1)
			# XXX FIX THIS!
			# self.pkgset = read_pickle(self.pickle)
			#self.announce("done")
			self.announce("failed")

		self.announce("resolving dependencies...", noeol=1)
		self.pkgset.resolve()
		self.announce("done")
		
		self.highlight = self.deps+self.revdeps

		if self.deps:
			deps = []
			for i in range(len(self.deps)):
				rels = self.pkgset.who_provides_any(self.deps[i])
				if not rels:
					raise DeepManagerOptionError, \
						  "no relation requiring %s was found" % self.deps[i]
				deps.extend(rels)
				self.highlight.extend(rels)
			self.deps = deps
		if self.revdeps:
			revdeps = []
			for i in range(len(self.revdeps)):
				rels = self.pkgset.who_requires_any(self.revdeps[i])
				if not rels:
					raise DeepManagerOptionError, \
						  "no package providing %s was found" % self.revdeps[i]
				revdeps.extend(rels)
				self.highlight.extend(rels)
			self.revdeps = revdeps

		self.announce("initializing objects...", noeol=1)
		for pkg in self.pkgset:
			self.set_defaults(pkg)
			map(self.set_defaults, pkg.provides)
			map(self.set_defaults, pkg.requires)
			map(self.set_defaults, pkg.conflicts)
			map(self.set_defaults, pkg.obsoletes)
		self.announce("done")

		if self.all_deps or self.all_revdeps:
			for pkg in self.pkgset:
				if pkg.name in self.exclude:
					continue
				self.indent = 0
				self.depth = 0
				self.written = []
				self.links = []

				self.announce("resetting packages...", noeol=1)
				for rpkg in self.pkgset:
					if hasattr(rpkg, "written_deps"):
						del rpkg.written_deps
					if hasattr(rpkg, "written_revdeps"):
						del rpkg.written_revdeps
				self.announce("done")

				self.announce("processing dependencies of %s..." % str(pkg),
							  noeol=1)
				if self.all_deps:
					self.process_pkg_deps(pkg)
				if self.all_revdeps:
					self.process_pkg_revdeps(pkg)
				self.announce("done")

				self.announce("writting graph...", noeol=1)
				self.file = open(self.output % {"name":pkg.name}, 'w')
				self.write("digraph G {")
				self.indent += 1
				self.write("rankdir=LR;")
				self.write_links()
				self.indent -= 1
				self.write("}")
				self.file.close()
				self.announce("done")
		else:
			self.indent = 0
			self.depth = 0
			self.written = []
			self.links = []

			self.announce("processing requested information...", noeol=1)
			for rel in self.deps:
				self.process_provrel_deps(rel)
			for rel in self.revdeps:
				self.process_reqrel_revdeps(rel)
			self.announce("done")

			self.announce("writting graph...", noeol=1)
			self.file = open(self.output, 'w')
			self.write("digraph G {")
			self.indent += 1
			self.write("rankdir=LR;")
			self.write_links()
			self.indent -= 1
			self.write("}")
			self.file.close()
			self.announce("done")

		self.announce("finished!")
	
	def add_link(self, req=None, prov=None):
		self.links.append((req, prov))
	
	def process_pkg_deps(self, pkg):
		if self.depth+1 > self.maxdepth or hasattr(pkg, "written_deps"):
			return
		self.depth += 1
		pkg.written_deps = 1
		for rel in pkg.requires:
			if not rel.relations:
				self.add_link(req=rel, prov=None)
			else:
				for provrel in rel.relations:
					if provrel.package.name not in self.exclude:
						self.add_link(req=rel, prov=provrel)
						self.process_pkg_deps(provrel.package)
		self.depth -= 1
	
	def process_pkg_revdeps(self, pkg):
		if self.depth+1 > self.maxdepth or hasattr(pkg, "written_revdeps"):
			return
		self.depth += 1
		pkg.written_revdeps = 1
		for rel in pkg.provides:
			for reqrel in rel.relations:
				if reqrel.package.name not in self.exclude:
					self.add_link(req=reqrel, prov=rel)
					self.process_pkg_revdeps(reqrel.package)
		self.depth -= 1
	
	def process_provrel_deps(self, rel):
		self.depth += 1
		self.add_link(req=None, prov=rel)
		self.process_pkg_deps(rel.package)
		self.depth -= 1
	
	def process_reqrel_revdeps(self, rel):
		self.depth += 1
		self.add_link(req=rel, prov=None)
		if rel.relations:
			for provrel in rel.relations:
				if provrel.name not in self.exclude:
					self.add_link(req=rel, prov=provrel)
		self.process_pkg_revdeps(rel.package)
		self.depth -= 1
	
	def write_links(self):
		relhash = self.pkgset.get_provides_hash()
		for req, prov in self.links:
			orig, dest = None, None
			origdestattr = Attr()
			if self.show_alldeps \
			   or self.show_filedeps and (req or prov).name[0] == "/":
				writerels = 1
			else:
				writerels = 0
			if req:
				linkattr = Attr()
				if not req.relations:
					req.attr.color = "red"
					linkattr.color = "red"
				if not req.flags & PKG_EXPLICIT:
					origdestattr.color = "blue"
				if writerels:
					self.write_link([req.package, req], linkattr)
					orig = req
				else:
					orig = req.package
			if prov:
				linkattr = Attr()
				if writerels and (prov.name != prov.package.name
								  or prov.name in self.highlight):
					self.write_link([prov, prov.package], linkattr)
					dest = prov
				else:
					dest = prov.package
				if len(self.pkgset.who_provides_rel(prov, relhash)) > 1:
					origdestattr.color = "cyan"
			if orig and dest and orig.nodename != dest.nodename:
				self.write_link([orig, dest], origdestattr)
	
	def write_link(self, nodes, attr=None):
		tuple = (" -> ".join(["\"%s\""%x.nodename for x in nodes]), str(attr))
		if tuple not in self.written:
			map(self.write_node, nodes)
			self.written.append(tuple)
			self.file.write('\t'*self.indent)
			self.file.write("%s%s;\n" % tuple)
	
	def write_node(self, node):
		tuple = (node.nodename, str(node.attr))
		if tuple not in self.written:
			self.written.append(tuple)
			self.write("\"%s\"%s;" % tuple)
	
	def write(self, line):
		if self.indent:
			self.file.write('\t'*self.indent)
		self.file.write(line+"\n")

	def set_defaults(self, obj):
		if not hasattr(obj, "attr"):
			obj.attr = Attr()
			if isinstance(obj, Relation):
				if obj.flags&PKG_REQUIRES and not obj.relations:
					obj.attr.color = "red"
					obj.attr.style = "filled"
				elif obj.name in self.highlight:
					obj.attr.color = "green"
					obj.attr.style = "filled"
				obj.attr.shape = "box"
				obj.nodename = "(%s)" % str(obj)
			else:
				if self.show_versions:
					obj.nodename = str(pkg)
				else:
					obj.nodename = obj.name
				if obj.name in self.highlight:
					obj.attr.color = "green"
					obj.attr.style = "filled"
				obj.attr.URL = "\"%s.html\"" % obj.name

class Attr:
	def __init__(self, **kw):
		self.__dict__.update(kw)
	
	def __str__(self):
		attr = " ".join(["=".join(x) for x in self.__dict__.items()])
		if attr:
			return " [%s]" % attr
		else:
			return ""

# vim:nowrap:sw=4:ts=4
