"""deepmanager.errors

This module was originally part of distutils.

Provides exceptions used by the DeepManager modules.  Note that DeepManager
modules may raise standard exceptions; in particular, SystemExit is
usually raised for errors that are obviously the end-user's fault
(eg. bad command-line arguments).

This module safe to use in "from ... import *" mode; it only exports
symbols whose names start with "DeepManager" and end with "Error"."""

# created 1999/03/03, Greg Ward

__revision__ = "$Id: errors.py,v 1.10 2000/07/30 01:03:31 gward Exp $"

class DeepManagerError (Exception):
    """The root of all DeepManager evil."""
    pass

class DeepManagerModuleError (DeepManagerError):
    """Unable to load an expected module, or to find an expected class
    within some module (in particular, command modules and classes)."""
    pass

class DeepManagerClassError (DeepManagerError):
    """Some command class (or possibly distribution class, if anyone
    feels a need to subclass Distribution) is found not to be holding
    up its end of the bargain, ie. implementing some part of the
    "command "interface."""
    pass

class DeepManagerGetoptError (DeepManagerError):
    """The option table provided to 'fancy_getopt()' is bogus."""
    pass

class DeepManagerArgError (DeepManagerError):
    """Raised by fancy_getopt in response to getopt.error -- ie. an
    error in the command line usage."""
    pass

class DeepManagerFileError (DeepManagerError):
    """Any problems in the filesystem: expected file not found, etc.
    Typically this is for problems that we detect before IOError or
    OSError could be raised."""
    pass

class DeepManagerOptionError (DeepManagerError):
    """Syntactic/semantic errors in command options, such as use of
    mutually conflicting options, or inconsistent options,
    badly-spelled values, etc.  No distinction is made between option
    values originating in the setup script, the command line, config
    files, or what-have-you -- but if we *know* something originated in
    the setup script, we'll raise DeepManagerSetupError instead."""
    pass

class DeepManagerSetupError (DeepManagerError):
    """For errors that can be definitely blamed on the setup script,
    such as invalid keyword arguments to 'setup()'."""
    pass

class DeepManagerPlatformError (DeepManagerError):
    """We don't know how to do something on the current platform (but
    we do know how to do it on some platform) -- eg. trying to compile
    C files on a platform not supported by a CCompiler subclass."""
    pass

class DeepManagerExecError (DeepManagerError):
    """Any problems executing an external program (such as the C
    compiler, when compiling C files)."""
    pass

class DeepManagerInternalError (DeepManagerError):
    """Internal inconsistencies or impossibilities (obviously, this
    should never be seen if the code is working!)."""
    pass

class DeepManagerTemplateError (DeepManagerError):
    """Syntax error in a file list template."""

class DeepManagerPackageError (DeepManagerError):
    """Invalid package processed."""
