from types import ListType
from UserList import UserList
import cPickle

PICKLEVERSION = 0

PKG_PROVIDES  = 1 << 0
PKG_REQUIRES  = 1 << 1
PKG_CONFLICTS = 1 << 2
PKG_OBSOLETES = 1 << 3
PKG_LESS      = 1 << 4
PKG_EQUAL     = 1 << 5
PKG_GREATER   = 1 << 6
PKG_EXPLICIT  = 1 << 7

PM = {}

def get_pm(type):
	"""Return package manager for the desired package type."""
	return PM.get(type)

class PackageManager:
	"""PackageManager base class for abstraction of package managers.

	Classes inheriting this class will hide all specific information for
	different package managers. DO NOT predict ANY specific algorithms
	outside of those classes.
	"""
	def __init__(self):
		if self.__class__ is PackageManager:
			raise RuntimeError, "PackageManager is an abstract class"

	def who_provides(self,
					  pkgset,
					  name,
					  epoch=None,
					  version=None,
					  release=None,
					  flags=0,
					  relhash=None):
		"""Return list of relations providing given information.
		
		This method is very important since it will be extensively used
		by the PackageSet for dependency resolution."""
		raise RuntimeError, \
			"PackageManager.who_provides() is a pure virtual method"
	
	def who_requires(self,
					 pkgset,
					 name,
					 epoch=None,
					 version=None,
					 release=None,
					 relhash=None):
		"""Return list of relations requiring given information."""
		raise RuntimeError, \
			"PackageManager.who_requires() is a pure virtual method"
	
	def who_conflicts(self,
					  pkgset,
					  name,
					  epoch=None,
					  version=None,
					  release=None,
					  relhash=None):
		"""Return list of relations conflicting with given information."""
		raise RuntimeError, \
			"PackageManager.who_conflicts() is a pure virtual method"

	def who_obsoletes(self,
					  pkgset,
					  name,
					  epoch=None,
					  version=None,
					  release=None,
					  relhash=None):
		"""Return list of relations obsoleting given information."""
		raise RuntimeError, \
			"PackageManager.who_obsoletes() is a pure virtual method"

class PackageSet(UserList):
	def __init__(self, pm):
		UserList.__init__(self)
		self.pm = pm
	
	def who_provides(self, *parms):
		"""Return list of relations providing given information."""
		return self.pm.who_provides(self, *parms)
	
	def who_requires(self, *parms):
		"""Return list of relations requiring given information."""
		return self.pm.who_requires(self, *parms)
	
	def who_conflicts(self, *parms):
		"""Return list of relations conflicting with given information."""
		return self.pm.who_conflicts(self, *parms)
	
	def who_obsoletes(self, *parms):
		"""Return list of relations obsoletes given information."""
		return self.pm.who_obsoletes(self, *parms)
	
	def who_provides_rel(self, rel, relhash=None):
		"""Return list of relations providing given relation."""
		return self.pm.who_provides(self,
									rel.name,
									rel.epoch,
									rel.version,
									rel.release,
									rel.flags,
									relhash)

	def who_requires_rel(self, rel, relhash=None):
		"""Return list of relations requiring given relation."""
		return self.pm.who_requires(self,
									rel.name,
									rel.epoch,
									rel.version,
									rel.release,
									relhash)
	
	def who_conflicts_rel(self, rel, relhash=None):
		"""Return list of relations conflicting with given relation."""
		return self.pm.who_conflicts(self,
									 rel.name,
									 rel.epoch,
									 rel.version,
									 rel.release,
									 relhash)
	
	def who_obsoletes_rel(self, rel, relhash=None):
		"""Return list of relations obsoleting given relation."""
		return self.pm.who_obsoletes(self,
									 rel.name,
									 rel.epoch,
									 rel.version,
									 rel.release,
									 relhash)
	
	def who_provides_any(self, name, relhash=None):
		"""Return list of relations providing named relation."""
		return self._who_relates_any(name, relhash, "provides")
	
	def who_requires_any(self, name, relhash=None):
		"""Return list of relations requiring named relation."""
		return self._who_relates_any(name, relhash, "requires")
	
	def who_conflicts_any(self, name, relhash=None):
		"""Return list of relations conflicting with named relation."""
		return self._who_relates_any(name, relhash, "conflicts")
	
	def who_obsoletes_any(self, name, relhash=None):
		"""Return list of relations conflicting with named relation."""
		return self._who_relates_any(name, relhash, "obsoletes")
	
	def _who_relates_any(self, name, relhash, attrname):
		if relhash == None:
			relhash = self._get_hash(attrname)
		if relhash.has_key(name):
			return relhash[name]
		return []

	def get_provides_hash(self):
		return self._get_hash("provides")

	def get_requires_hash(self):
		return self._get_hash("requires")

	def get_conflicts_hash(self):
		return self._get_hash("conflicts")
	
	def get_obsoletes_hash(self):
		return self._get_hash("obsoletes")
	
	def _get_hash(self, attrname):
		hash = {}
		for pkg in self:
			for rel in getattr(pkg, attrname):
				try:
					hash[rel.name].append(rel)
				except KeyError:
					hash[rel.name] = [rel]
		return hash
	
	def resolve(self, relhash=None):
		"""Link relations."""
		if relhash == None:
			relhash = self.get_provides_hash()
		for pkg in self:
			for rel in pkg.requires+pkg.conflicts+pkg.obsoletes:
				providers = self.who_provides_rel(rel, relhash)
				rel.relations = providers
				for provrel in providers:
					provrel.relations.append(rel)

class Package:
	def __init__(self, pm, name, epoch=None, version=None, release=None,
				 arch=None):
		self.pm = pm
		self.name = name
		self.epoch = epoch
		self.version = version
		self.release = release
		self.arch = arch
		self.requires = []
		self.provides = []
		self.conflicts = []
		self.obsoletes = []

	def __str__(self):
		if self.epoch:
			return "%s-%s:%s-%s" % (self.name, self.epoch, self.version,
									self.release)
		else:
			return "%s-%s-%s" % (self.name, self.version, self.release)

	def __repr__(self):
		return str(self)

class Relation:
	def __init__(self, package, name, epoch=None, version=None,
				 release=None, flags=0, relations=None):
		self.name = name
		self.epoch = epoch
		self.version = version
		self.release = release
		self.package = package
		self.flags = flags
		self.relations = relations or []
	
	def __str__(self):
		if self.flags & (PKG_LESS | PKG_EQUAL | PKG_GREATER):
			cmpstr = " "
			if self.flags & PKG_LESS:
				cmpstr = " <"
			elif self.flags & PKG_GREATER:
				cmpstr = " >"
			if self.flags & PKG_EQUAL:
				cmpstr += "="
		else:
			cmpstr = ""
		if self.epoch:
			verstr = " %s:%s" % (self.epoch, self.version)
		elif self.version:
			verstr = " %s" % self.version
		else:
			verstr = ""
		if self.release:
			verstr = "%s-%s" % (verstr, self.release)
		return "%s%s%s" % (self.name, cmpstr, verstr)

	def __repr__(self):
		return str(self)

# vim:ts=4:sw=4
